% Copyright (c) 2016, Carlos Ciller 
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
% 
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in
%       the documentation and/or other materials provided with the distribution
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

function [flow,labels] = graphcutMRISuper(alpha, lambda, volume_intensity_T1, volume_intensity_T2, T1_tumor_std, T2_tumor_std, votes, super_ids, normalization_max)
    intensity_T1 = double(volume_intensity_T1);
    intensity_T2 = double(volume_intensity_T2);

    intensity_T1 = reshape(intensity_T1,1,[]);
    intensity_T2 = reshape(intensity_T2,1,[]);

    standard_deviation_data_T1 =  T1_tumor_std;
    standard_deviation_data_T2 = T2_tumor_std;

    %normalize votes to 0..1
    votes = votes ./ normalization_max;

    if (min(votes(:)) < 0) || (max(votes(:)) > 1)
      throw(MException('graphcutMRI:wrongnormalization','Votes are not normalized. Possibly wrong number of trees.'));
    end

%     disp('building graph');

    %% Construct graph
    tic;
    [Adjacencies, Weights] = createConnectivitySupervoxel(super_ids);
    toc

    % Variation ND Paper (Boykov) for B_p-q (Boundary similarity)
%     alpha   =   0.5; % Weighting similarity T2
    beta = 1.0-alpha; % Weighting similarity T1

    meanSupervoxelT1 = zeros(max(super_ids(:)) - min(super_ids(:))+1, 1);
    meanSupervoxelT2 = zeros(max(super_ids(:)) - min(super_ids(:))+1, 1);
    meanVotes = zeros(numel(meanSupervoxelT1), 1);

    for i = min(super_ids(:)):max(super_ids(:))
        idx = super_ids(:) == i;
        meanSupervoxelT1(i+1) = mean(intensity_T1(idx));
        meanSupervoxelT2(i+1) = mean(intensity_T2(idx));
        meanVotes(i+1) = mean(votes(idx));
    end

    V_T1 = exp(-((meanSupervoxelT1(Adjacencies(:,1))-meanSupervoxelT1(Adjacencies(:,2))).^2)./(2*standard_deviation_data_T1^2))+eps; % T1
    V_T2 = exp(-((meanSupervoxelT2(Adjacencies(:,1))-meanSupervoxelT2(Adjacencies(:,2))).^2)./(2*standard_deviation_data_T2^2))+eps; % T2
    V = ((alpha * V_T1 + beta * V_T2)./Weights');

    N = numel(meanSupervoxelT2);
    A = sparse(Adjacencies(:,1), Adjacencies(:,2), V', N, N);

    %% Weighting factor - Graph Cut

    % Lambda factor to multiply probability map (not normalized)
    labels_weight_healthy = double(zeros(numel(meanSupervoxelT2),1));
    labels_weight_tumor = double(zeros(numel(meanSupervoxelT2),1));

    % tumor labels
    labels_weight_tumor(:) = -lambda * log(eps+double(meanVotes(:)));% negative-log-likelihood
    % healthy labels
    labels_weight_healthy(:) = -lambda * log(eps+double(1-meanVotes(:))); %negative-log-likelihood

    labels_weight = [labels_weight_tumor, labels_weight_healthy];

    T=sparse(labels_weight);

%     disp('calculating maximum flow');

    tic;
      [flow, idx_labels] = maxflow(A,T);
    toc

    labels = zeros(size(volume_intensity_T1));

    for i = min(super_ids(:)):max(super_ids(:))
      idx = super_ids(:) == i;
      labels(idx) = idx_labels(i+1);
    end
    
    subplot(2, 2, 1);
    imshow(squeeze(votes(:, :, 50)), []);
    subplot(2,2,2);
    imshow(squeeze(labels(:, :, 50)), []);
    subplot(2,2,3);
    imshow(squeeze(volume_intensity_T1(:,:,50)), []);
    subplot(2,2,4);
    imshow(squeeze(super_ids(:, :, 50)), []);
    pause;

end
